@@ -92,7 +92,7 @@ for (i in 2:niter) {
   curDelta <- postDelta[i-1,]; curTheta1 <- postTheta1[i-1,]
   for (j in 1:p1) {
     ej <- e+curTheta1[j]*x[,j]
-    newval <- MHTheta1emom(ej,j=j,delta=curDelta,theta1=curTheta1,phi=postPhi[i-1],tau=tau,xj=x[,j],modelPrior=modelPrior)
+    newval <- MHTheta1emom(ej,j=j,delta=curDelta,theta1=curTheta1,phi=postPhi[i-1],tau=tau,xj=x[,j],padj=p2,modelPrior=modelPrior)
     curDelta[j] <- newval$delta; curTheta1[j] <- newval$theta1
     if (newval$accept) e <- ej - curTheta1[j]*x[,j]   #Update residuals
   }
@@ -214,7 +214,7 @@ for (i in 2:niter) {
   curDelta <- postDelta[i-1,]; curTheta1 <- postTheta1[i-1,]
   for (j in 1:p1) {
     ej <- e+curTheta1[j]*x[,j]
-    newval <- MHTheta1emom(ej,j=j,delta=curDelta,theta1=curTheta1,phi=1,tau=tau,xj=x[,j],modelPrior=modelPrior)
+    newval <- MHTheta1emom(ej,j=j,delta=curDelta,theta1=curTheta1,phi=1,tau=tau,xj=x[,j],padj=p2,modelPrior=modelPrior)
     curDelta[j] <- newval$delta; curTheta1[j] <- newval$theta1
     if (newval$accept) e <- ej - curTheta1[j]*x[,j]   #Update residuals
   }
@@ -274,7 +274,7 @@ proposaleMOM <- function(m,S,phi,tau,e,xj,m1,nu) {
 }
 
     
-MHTheta1emom <- function(e,j,delta,theta1,phi,tau,xj,modelPrior) {
+MHTheta1emom <- function(e,j,delta,theta1,phi,tau,xj,padj,modelPrior) {
   #MH step to simulate (delta[j], theta1[j]) from its posterior given the data, delta[-j], theta1[-j], theta2 and phi parameters
   # Input
   # - e: partial residuals, i.e. y - predicted y given all covariates except covariate j
@@ -290,11 +290,16 @@ MHTheta1emom <- function(e,j,delta,theta1,phi,tau,xj,modelPrior) {
   # - theta1: new value for theta1[j]
   # - accept: logical variable indicated whether proposed new value has been accepted or not
   #Propose
+  pcur <- sum(delta)
   m1 <- emomMargKuniv(y=e, x=xj, phi=phi, tau=tau, logscale=TRUE)
   logbf <- sum(dnorm(e,0,sd=sqrt(phi),log=TRUE)) - m1
   delta0 <- delta1 <- delta; delta0[j] <- FALSE; delta1[j] <- TRUE
   logpratio01 <- modelPrior(delta0) - modelPrior(delta1)
-  p <- 1/(1 + exp(logbf+logpratio01))
+  if ((delta[j]==0) & ((pcur+padj) >= length(e))) {
+      p <- 0
+  } else {
+      p <- 1/(1 + exp(logbf+logpratio01))
+  }
   deltaProp <- rbinom(n=1,size=1,prob=p)
   nu <- sqrt(ifelse(is.matrix(e),nrow(e),length(e)))
   #Acceptance prob
