@@ -49,7 +49,8 @@ valid_msPriorSpec <- function(object) {
            coefficients = {
                valid_coef_prior_distrs <- c("pMOM",
                                             "piMOM",
-                                            "peMOM")
+                                            "peMOM",
+                                            "zellner")
                found <- object@priorDistr %in% valid_coef_prior_distrs
                if (!found) {
                    msg <- c(msg,
@@ -185,6 +186,10 @@ emomprior <- function(tau, tau.adj=10^6) {
     new("msPriorSpec", priorType="coefficients", priorDistr="peMOM", priorPars=c(tau=tau, tau.adj=tau.adj))
 }
 
+zellnerprior <- function(tau, tau.adj=10^6) {
+    new("msPriorSpec", priorType="coefficients", priorDistr="zellner", priorPars=c(tau=tau, tau.adj=tau.adj))
+}
+
 modelunifprior <- function() {
     new("msPriorSpec",priorType='modelIndicator',priorDistr='uniform',priorPars=double(0))
 }
