@@ -240,8 +240,8 @@ void choldc_inv(double **a, int n, double **aout, bool *posdef); //Inverse of ch
 void cholS_inv(double **cholS, int n, double **cholSinv); //Inverse of cholS
 void choldc_inv_internal(double **cholS, int n); 
 double choldc_det(double **chols, int n); //Determinant of a symmetric def+ using its Cholesky decomp
-void inv_posdef(double **a, int n, double **aout); //Inverse of a symmetric, positive definite matrix
-void inv_posdef_upper(double **a, int n, double **aout); //Same but only returns upper triangular elements
+void inv_posdef(double **a, int n, double **aout, bool *posdef); //Inverse of a symmetric, positive definite matrix
+void inv_posdef_upper(double **a, int n, double **aout, bool *posdef); //Same but only returns upper triangular elements
 void invdet_posdef(double **a, int n, double **aout, double *det_a); //Inverse and determinant of positive def matrix
 void inv_posdef_chol(double **invchol, int n, double **aout); //Inverse given cholesky decomposition
 
